NT_RollResults = {};
NT_RollHistory = {};

NT_PartyStatus = 0;
NT_InstanceStatus = "";

local function TrimSpecial(str)
    return str
        :gsub("%|c%x%x%x%x%x%x%x%x", "")
        :gsub("%|r", "");
end

local function PrintToChat(msg, channel)
    if channel then
        local filtered = "";
        while msg:len() > 0 do
            local p1, p2, p3 = msg:match("^(.-)(%|c%x%x%x%x%x%x%x%x%|Hitem:.-%|h%|r)(.*)$");
            if not p1 then
                filtered = filtered .. TrimSpecial(msg);
                break;
            end
            filtered = filtered .. TrimSpecial(p1) .. p2;
            msg = p3;
        end

        SendChatMessage(filtered, channel);
    else
        print(msg);
    end
end

local function GetName(name)
    local i_name = name:match("^%|3%-%d+%((.+)%)$");
    if i_name then
        return i_name;
    end
    return name;
end

local function IsInParty()
    if GetNumPartyMembers() > 0 then
        return true;
    end
    return false;
end

local function IsInLFGParty()
    if GetLFGMode() == "lfgparty" then
        return true;
    end
    return false;
end

local function IsInRaid()
    if GetNumRaidMembers() > 0 then
        return true;
    end
    return false;
end

local function OnRoll(name, item, need)
    local res = NT_RollResults[item];
    if not res then
        res = {};
        NT_RollResults[item] = res;
    end

    res[name] = { ["need"] = need };

    if need == "Need" and name ~= UnitName("player") then
        PrintToChat((NT_LOCALE["MESSAGES"]["ROLL_NEED"]):format(item, name, NT_LOCALE["NEED"][need]));
    end
end

local function OnResult(name, item, need, roll)
    local res = NT_RollResults[item];
    if not res then
        res = {};
        NT_RollResults[item] = res;
    end

    res[name] = { ["need"] = need, ["roll"] = roll };
end

local function OnWin(name, item)
    local res = NT_RollResults[item];
    if not res then
        res = {};
        NT_RollResults[item] = res;
    end

    local need, roll = "Unknown", "?";

    if res[name] then
        if res[name]["need"] then
            need = res[name]["need"];
        end
        if res[name]["roll"] then
            roll = res[name]["roll"];
        end
    end

    if (name == UnitName("player")) then
        PrintToChat((NT_LOCALE["MESSAGES"]["WIN_YOU"]):format(item, NT_LOCALE["NEED"][need], roll));

    elseif (need == "Need") then
        PrintToChat((NT_LOCALE["MESSAGES"]["WIN_NEED"]):format(item, name, NT_LOCALE["NEED"][need], roll));

    else
        PrintToChat((NT_LOCALE["MESSAGES"]["WIN_PARTY"]):format(item, name, NT_LOCALE["NEED"][need], roll));
    end

    table.insert(NT_RollHistory, { ["item"] = item, ["name"] = name, ["need"] = need, ["roll"] = roll });
    while # NT_RollHistory > 20 do
        table.remove(NT_RollHistory, 1);
    end

    NT_RollResults[item] = nil;
end

local function OnPass(name, item)
    local res = NT_RollResults[item];
    if not res then
        res = {};
        NT_RollResults[item] = res;
    end

    res[name] = { ["need"] = "Pass" };
end

local function OnDiscard(item)
    local res = NT_RollResults[item];
    if not res then
        res = {};
        NT_RollResults[item] = res;
    end

    PrintToChat((NT_LOCALE["MESSAGES"]["PASS_PARTY"]):format(item));

    table.insert(NT_RollHistory, { ["item"] = item });
    while # NT_RollHistory > 20 do
        table.remove(NT_RollHistory, 1);
    end

    NT_RollResults[item] = nil;
end

local function ClearHistory()
    NT_RollResults = {};
    NT_RollHistory = {};
end

local function ShowHistoryReport(channel)
    if # NT_RollHistory == 0 then
        PrintToChat(NT_LOCALE["MESSAGES"]["HISTORY_EMPTY"], channel);
        return;
    end

    PrintToChat(NT_LOCALE["MESSAGES"]["HISTORY_HEADER"], channel);

    local i, history;
    for i, history in pairs(NT_RollHistory) do
        local line, format;
        if history["name"] then
            if history["need"] == "Need" and history["name"] ~= UnitName("player") then
                format = NT_LOCALE["MESSAGES"]["HISTORY_NEED"];
            else
                format = NT_LOCALE["MESSAGES"]["HISTORY_WIN"];
            end
            line = format:format(i, history["item"], history["name"],
                NT_LOCALE["NEED"][history["need"]], history["roll"]);
        else
            format = NT_LOCALE["MESSAGES"]["HISTORY_PASS"];
            line = format:format(i, history["item"]);
        end
        PrintToChat(line, channel);
    end
end

local function ShowRollReport(channel)
end

local function ParseLootMessage(msg)
    local i, match;
    for i, match in pairs(NT_LOCALE["CHAT_MATCH"]) do
        local p1, p2, p3, p4 = msg:match("^" .. match[1] .. "$");
        if p1 then
            if match[2] == "RESULT_WIN" then
                OnResult(GetName(p1), p3, NT_LOCALE["NEED_MATCH"][p2], p4);

            elseif match[2] == "RESULT_PASS" then
                OnDiscard(p1);

            elseif match[2] == "ROLL_YOU" then
                OnRoll(UnitName("player"), p1, NT_LOCALE["NEED_MATCH"][p2]);

            elseif match[2] == "ROLL_PARTY" then
                OnRoll(GetName(p2), p1, NT_LOCALE["NEED_MATCH"][p3]);

            elseif match[2] == "PASS_YOU" then
                OnPass(UnitName("player"), p1);

            elseif match[2] == "PASS_PARTY" then
                OnPass(GetName(p1), p2);

            elseif match[2] == "WIN_YOU" then
                OnWin(UnitName("player"), p1);

            elseif match[2] == "WIN_PARTY" then
                OnWin(GetName(p1), p2);
            end;
            break;
        end
    end
end

local function Ru_ParseLootMessage(msg)
    local i, match;
    for i, match in pairs(NT_LOCALE["CHAT_MATCH"]) do
        local p1, p2, p3, p4 = msg:match("^" .. match[1] .. "$");
        if p1 then
            if match[2] == "RESULT_WIN" then
                OnResult(GetName(p1), p3, NT_LOCALE["NEED_MATCH"][p2], p4);

            elseif match[2] == "RESULT_PASS" then
                OnDiscard(p1);

            elseif match[2] == "ROLL_YOU" then
                OnRoll(UnitName("player"), p1, NT_LOCALE["NEED_MATCH"][p2]);

            elseif match[2] == "ROLL_PARTY" then
                OnRoll(GetName(p2), p1, NT_LOCALE["NEED_MATCH"][p3]);

            elseif match[2] == "PASS_YOU" then
                OnPass(UnitName("player"), p1);

            elseif match[2] == "PASS_PARTY" then
                OnPass(GetName(p1), p2);

            elseif match[2] == "WIN_YOU" then
                OnWin(UnitName("player"), p1);

            elseif match[2] == "WIN_PARTY" then
                OnWin(GetName(p1), p2);
            end;
            break;
        end
    end
end

local function CheckPartyStatus()
    if IsInRaid() then
        if NT_PartyStatus < 2 then
            ClearHistory();
        end
        NT_PartyStatus = 2;
    elseif IsInParty() then
        if NT_PartyStatus < 1 then
            ClearHistory();
        end
        NT_PartyStatus = 1;
    else
        NT_PartyStatus = 0;
    end
end

local function CheckInstanceStatus()
    local name, type = GetInstanceInfo();
    if type ~= "none" then
        if (NT_InstanceStatus ~= name) then
            ClearHistory();
        end
        NT_InstanceStatus = name;
    end
end

function NT_OnLoad()
    local frame = CreateFrame("Frame", "NeedjaTracker", UIParent);

    frame:SetScript("OnEvent", NT_OnEvent);

    frame:RegisterEvent("VARIABLES_LOADED");
    frame:RegisterEvent("CHAT_MSG_LOOT");
    frame:RegisterEvent("PARTY_MEMBERS_CHANGED");
    frame:RegisterEvent("RAID_ROSTER_UPDATE");
    frame:RegisterEvent("PLAYER_ENTERING_WORLD");

    ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", NT_ChatFilter);
end

function NT_OnEvent(self, event, ...)
    local arg_1 = ...;

    if event == "VARIABLES_LOADED" then
        CheckPartyStatus();
        CheckInstanceStatus();

    elseif event == "CHAT_MSG_LOOT" then
        local locale = GetLocale();
        if locale == "ruRU" then
            Ru_ParseLootMessage(arg_1);
        else
            ParseLootMessage(arg_1);
        end

    elseif event == "PARTY_MEMBERS_CHANGED" or event == "RAID_ROSTER_UPDATE" or
        event == "PLAYER_ENTERING_WORLD" then

        CheckPartyStatus();
        CheckInstanceStatus();
    end
end

function NT_ChatFilter(self, event, msg)
    local i, match;
    for i, match in pairs(NT_LOCALE["CHAT_MATCH"]) do
        if msg:match(match[1]) then
            return true;
        end
    end
    return false;
end

SLASH_NEEDJATRACKER1 = "/needjatracker";
SLASH_NEEDJATRACKER2 = "/nt";

function SlashCmdList.NEEDJATRACKER(msg)
    local channel;
    if (msg:find("party")) then
        if not IsInParty() then
            return;
        end
        channel = "PARTY";

    elseif (msg:find("raid")) then
        if not IsInRaid() then
            return;
        end
        channel = "RAID";
    end;

    if msg:find("report") then
        ShowRollReport(channel);

    elseif msg:find("history") then
        ShowHistoryReport(channel);

    elseif msg:find("clear") then
        ClearHistory();
    else
        local i;
        for i = 1, 100 do
            local help = NT_LOCALE["MESSAGES"]["HELP_LINE_" .. i];
            if help then
                PrintToChat(help);
            end
       end;
    end
end

NT_OnLoad();
